Package: msigdb
Title: An ExperimentHub Package for the Molecular Signatures Database
        (MSigDB)
Version: 1.21.0
Authors@R: c(
            person(given = "Dharmesh D.",
                   family = "Bhuva",
                   role = c("aut", "cre"),
                   email = "bhuva.d@wehi.edu.au",
                   comment = c(ORCID = "0000-0002-6398-9157")),
            person(given = "Gordon K.",
                   family = "Smyth",
                   role = c("aut"),
                   comment = c(ORCID = "0000-0001-9221-2892")),
            person(given = "Alexandra",
                   family = "Garnham",
                   role = c("aut"),
                   comment = c(ORCID = "0000-0002-8312-8450"))
            )
Description: This package provides the Molecular Signatures Database (MSigDB) in
    a R accessible objects. Signatures are stored in GeneSet class objects form
    the GSEABase package and the entire database is stored in a GeneSetCollection
    object. These data are then hosted on the ExperimentHub. Data used in this 
    package was obtained from the MSigDB of the Broad Institute. Metadata for each
    gene set is stored along with the gene set in the GeneSet class object.
biocViews: ExperimentHub, Homo_sapiens_Data, Mus_musculus_Data
License: CC BY 4.0
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
Depends: R (>= 4.1)
Imports: ExperimentHub, utils, GSEABase, org.Mm.eg.db, org.Hs.eg.db,
        AnnotationDbi, methods, stats, AnnotationHub
Suggests: singscore, vissE, knitr, prettydoc, BiocStyle, rmarkdown,
        testthat (>= 3.0.0), BiocFileCache, GO.db, stringr, limma
URL: https://davislaboratory.github.io/msigdb
BugReports: https://github.com/DavisLaboratory/msigdb/issues
NeedsCompilation: no
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/msigdb
git_branch: devel
git_last_commit: 1c15606
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-30
Packaged: 2026-04-30 15:57:04 UTC; biocbuild
Author: Dharmesh D. Bhuva [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6398-9157>),
  Gordon K. Smyth [aut] (ORCID: <https://orcid.org/0000-0001-9221-2892>),
  Alexandra Garnham [aut] (ORCID:
    <https://orcid.org/0000-0002-8312-8450>)
Maintainer: Dharmesh D. Bhuva <bhuva.d@wehi.edu.au>

# Generated by roxygen2: do not edit by hand

export(appendKEGG)
export(getIMEX)
export(getIMEXVersions)
export(getMsigIdType)
export(getMsigOrganism)
export(getMsigdb)
export(getMsigdbIDF)
export(getMsigdbVersions)
export(listCollections)
export(listSubCollections)
export(subsetCollection)

library(plyr)
library(stringr)
library(org.Hs.eg.db)
library(org.Mm.eg.db)
library(tidyverse)
library(BiocFileCache)

imex_date = '2022-07-11'
bfc = BiocFileCache()
imex_path = bfcrpath(bfc, sprintf('ftp://ftp.ebi.ac.uk/pub/databases/intact/%s/psimitab/intact-micluster.txt', imex_date))

#----download PPI----
#process human and mouse PPI
ppi = read.csv(imex_path, sep = '\t', header = TRUE)
ppi = ppi |> 
  #find interactions determined within the same species
  filter(Taxid.interactor.A == Taxid.interactor.B) |> 
  #extract values
  mutate(
    Taxid = str_extract(Taxid.interactor.A, '[0-9]+')
  ) |> 
  #choose human and mouse only
  filter(Taxid %in% c('9606', '10090')) |> 
  mutate(
    InteractionType = gsub('psi-mi:MI:[0-9]+\\(|\\)', '', Interaction.type.s.),
    DetectionMethod = gsub('psi-mi:MI:[0-9]+\\(|\\)', '', Interaction.detection.method.s.)
  ) |>
  #extract values
  mutate(
    Confidence = as.numeric(str_remove(Confidence.value.s., '^.*:')),
    InteractorA = str_remove(Alt..ID.s..interactor.A, '^.*:'),
    InteractorB = str_remove(Alt..ID.s..interactor.B, '^.*:')
  ) |> 
  #map uniprot to entrez
  mutate(
    EntrezA = case_when(
      Taxid == '9606' ~ mapIds(org.Hs.eg.db, InteractorA, 'ENTREZID', 'UNIPROT'),
      Taxid == '10090' ~ mapIds(org.Mm.eg.db, InteractorA, 'ENTREZID', 'UNIPROT'),
    ),
    EntrezB = case_when(
      Taxid == '9606' ~ mapIds(org.Hs.eg.db, InteractorB, 'ENTREZID', 'UNIPROT'),
      Taxid == '10090' ~ mapIds(org.Mm.eg.db, InteractorB, 'ENTREZID', 'UNIPROT'),
    )
  ) |> 
  filter(!is.na(EntrezA) & !is.na(EntrezB)) |> 
  #map uniprot to symbol
  mutate(
    SymbolA = case_when(
      Taxid == '9606' ~ mapIds(org.Hs.eg.db, EntrezA, 'SYMBOL', 'ENTREZID'),
      Taxid == '10090' ~ mapIds(org.Mm.eg.db, EntrezA, 'SYMBOL', 'ENTREZID'),
    ),
    SymbolB = case_when(
      Taxid == '9606' ~ mapIds(org.Hs.eg.db, EntrezB, 'SYMBOL', 'ENTREZID'),
      Taxid == '10090' ~ mapIds(org.Mm.eg.db, EntrezB, 'SYMBOL', 'ENTREZID'),
    )
  ) |> 
  distinct() |> 
  #summarise information for each pair
  select(
    InteractorA,
    InteractorB,
    EntrezA,
    EntrezB,
    SymbolA,
    SymbolB,
    Taxid,
    InteractionType,
    DetectionMethod,
    Confidence
  ) |> 
  mutate(Inferred = FALSE)

#split human and mouse
ppi_hs = ppi[ppi$Taxid %in% '9606', ]
ppi_mm = ppi[ppi$Taxid %in% '10090', ]

#----translate using homologs----
ppi_hs_hcop = ppi_mm |>
  left_join(msigdb:::hcop, by = c('EntrezA' = 'mouse_entrez_gene')) |> 
  left_join(msigdb:::hcop, by = c('EntrezB' = 'mouse_entrez_gene')) |> 
  ungroup() |>
  select(!(EntrezA:EntrezB)) |>
  rename(EntrezA = human_entrez_gene.x, EntrezB = human_entrez_gene.y) |>
  mutate(
    SymbolA = mapIds(org.Hs.eg.db, EntrezA, 'SYMBOL', 'ENTREZID'),
    SymbolB = mapIds(org.Hs.eg.db, EntrezB, 'SYMBOL', 'ENTREZID'),
    Taxid = '9606',
    Inferred = TRUE
  ) |>
  distinct() |> 
  as.data.frame()
ppi_hs = rbind(ppi_hs, ppi_hs_hcop)

ppi_mm_hcop = ppi_hs |>
  left_join(msigdb:::hcop, by = c('EntrezA' = 'human_entrez_gene')) |> 
  left_join(msigdb:::hcop, by = c('EntrezB' = 'human_entrez_gene')) |> 
  ungroup() |>
  select(!(EntrezA:EntrezB)) |>
  rename(EntrezA = mouse_entrez_gene.x, EntrezB = mouse_entrez_gene.y) |>
  mutate(
    SymbolA = mapIds(org.Mm.eg.db, EntrezA, 'SYMBOL', 'ENTREZID'),
    SymbolB = mapIds(org.Mm.eg.db, EntrezB, 'SYMBOL', 'ENTREZID'),
    Taxid = '10090',
    Inferred = TRUE
  ) |>
  distinct() |> 
  as.data.frame()
ppi_mm = rbind(ppi_mm, ppi_mm_hcop)
rm(ppi_mm_hcop, ppi_hs_hcop)

#----remove redundancry----
ppi = rbind(ppi_hs, ppi_mm) |> distinct()

ppi = ppi |> 
  mutate(switch = as.numeric(EntrezA) > as.numeric(EntrezB)) |> 
  mutate(
    tmpB = EntrezB,
    EntrezB = if_else(switch, EntrezA, EntrezB),
    EntrezA = if_else(switch, tmpB, EntrezA),
    tmpB = SymbolB,
    SymbolB = if_else(switch, SymbolA, SymbolB),
    SymbolA = if_else(switch, tmpB, SymbolA),
    tmpB = InteractorB,
    InteractorB = if_else(switch, InteractorA, InteractorB),
    InteractorA = if_else(switch, tmpB, InteractorA)
  ) |> 
  distinct() |> 
  group_by(EntrezA, EntrezB, Taxid) |> 
  summarise(
    InteractorA = paste(unique(InteractorA), collapse = '|'),
    InteractorB = paste(unique(InteractorB), collapse = '|'),
    SymbolA = paste(unique(SymbolA), collapse = '|'),
    SymbolB = paste(unique(SymbolB), collapse = '|'),
    InteractionType = paste(InteractionType, collapse = '|'),
    DetectionMethod = paste(DetectionMethod, collapse = '|'),
    Confidence = max(Confidence),
    Inferred = all(Inferred)
  ) |> 
  ungroup()

#remove duplicate evidence
ppi$InteractionType = sapply(str_split(ppi$InteractionType, '\\|'), function(x) {
  paste(unique(x), collapse = '|')
})
ppi$DetectionMethod = sapply(str_split(ppi$DetectionMethod, '\\|'), function(x) {
  paste(unique(x), collapse = '|')
})

saveRDS(ppi, file = sprintf('imex_hsmm_%s.rds', format(as.Date(imex_date), '%m%y')))

library(GSEABase)
library(BiocFileCache)
library(GO.db)
library(stringr)
library(reshape2)
library(limma)
library(org.Hs.eg.db)
library(org.Mm.eg.db)
library(foreach)
library(doSNOW)
library(iterators)

#file cache to download files to
bfc = BiocFileCache()

#getBroadSets that can deal with duplicate geneset names
getBroadSets2 <- function (uri, ..., membersId = c("MEMBERS_SYMBOLIZED", "MEMBERS_EZID")) 
{
  membersId <- match.arg(membersId)
  factories <- sapply(uri, GSEABase:::.BroadXMLNodeToGeneSet_factory, 
                      membersId = membersId)
  tryCatch({
    geneSets <- unlist(mapply(GSEABase:::.fromXML, uri, "//GENESET", 
                              factories, SIMPLIFY = FALSE, USE.NAMES = FALSE))
  }, error = function(err) {
    stop("'getBroadSets' failed to create gene sets:\n  ", 
         conditionMessage(err), call. = FALSE)
  })
  
  snames = sapply(geneSets, setName)
  if (any(duplicated(snames))) {
    #identify dups
    dups = snames[duplicated(snames)]
    uq = geneSets[!snames %in% dups]
    #merge dups
    dupmerged = lapply(dups, function(x) {
      x = geneSets[snames %in% x]
      geneIds(x[[1]]) = do.call(union, lapply(x, geneIds))
      return(x[[1]])
    })
    geneSets = c(uq, dupmerged)
  }
  
  GeneSetCollection(geneSets)
}

#function to download any given version of MSigDB
getMsigdbData <- function(msigdb_ver, old = FALSE) {
  msigdb_url = 'https://data.broadinstitute.org/gsea-msigdb/msigdb/release/___/msigdb_v___.xml'
  msigdb_url = gsub('___', msigdb_ver, msigdb_url)
  
  #download file
  msigpath = bfcrpath(bfc, msigdb_url)
  
  #replace C5:GO
  c5 = createC5OrgDb(org.Hs.egGO2ALLEGS, 'Homo sapiens', old)
  c5 = ez2symMsigdb(c5, org.Hs.eg.db)
  
  #----Symbols----
  #read genesets into a geneset collection
  msigdb = getBroadSets2(msigpath, membersId = 'MEMBERS_SYMBOLIZED')
  #remove KEGG (due to licenses)
  msigdb = GeneSetCollection(msigdb[!sapply(lapply(msigdb, collectionType), bcSubCategory) %in% 'CP:KEGG'])
  #remove archived
  msigdb = GeneSetCollection(msigdb[!sapply(lapply(msigdb, collectionType), bcCategory) %in% 'archived'])
  #remove GO
  gosubcat = c('GO:BP', 'GO:MF', 'GO:CC')
  msigdb = GeneSetCollection(msigdb[!sapply(lapply(msigdb, collectionType), bcSubCategory) %in% gosubcat])
  
  #gene set modifications
  msigdb = GeneSetCollection(lapply(msigdb, function(gs) {
    #add URLs
    urls(gs) = msigdb_url
    setVersion(gs) = new('Versions', msigdb_ver)
    return(gs)
  }))
  
  #add GO
  msigdb = c(msigdb, c5$sym)
  
  #remove empty gene sets
  msigdb_lengths = sapply(lapply(msigdb, geneIds), length)
  msigdb = GeneSetCollection(msigdb[msigdb_lengths > 0])
  msigdb.sym = msigdb
  
  #----Entrez IDs----
  #read genesets into a geneset collection
  msigdb = getBroadSets2(msigpath, membersId = 'MEMBERS_EZID')
  #remove KEGG (due to licenses)
  msigdb = GeneSetCollection(msigdb[!sapply(lapply(msigdb, collectionType), bcSubCategory) %in% 'CP:KEGG'])
  #remove archived
  msigdb = GeneSetCollection(msigdb[!sapply(lapply(msigdb, collectionType), bcCategory) %in% 'archived'])
  #remove GO
  gosubcat = c('GO:BP', 'GO:MF', 'GO:CC')
  msigdb = GeneSetCollection(msigdb[!sapply(lapply(msigdb, collectionType), bcSubCategory) %in% gosubcat])
  
  #gene set modifications
  msigdb = GeneSetCollection(lapply(msigdb, function(gs) {
    #convert idType
    gs@geneIdType = EntrezIdentifier()
    #add URLs
    urls(gs) = msigdb_url
    setVersion(gs) = new('Versions', msigdb_ver)
    return(gs)
  }))
  
  #add GO
  msigdb = c(msigdb, c5$ezid)
  
  #remove empty gene sets
  msigdb_lengths = sapply(lapply(msigdb, geneIds), length)
  msigdb = GeneSetCollection(msigdb[msigdb_lengths > 0])
  msigdb.ezid = msigdb
  
  return(list('sym' = msigdb.sym, 'ezid' = msigdb.ezid))
}

#function to convert the human MSigDB to mouse
createMmMsigdbData <- function(hsdb, old = FALSE) {
  #remove c1 and c5 genesets (these need to be replaced completely); except for HPO in c5
  rmgs = sapply(lapply(hsdb, collectionType), bcCategory) %in% c('c1', 'c5')
  rmgs[sapply(lapply(hsdb, collectionType), bcSubCategory) %in% 'HPO'] = FALSE
  mmdb = hsdb[!rmgs]
  
  #convert IDs using Ensembl homology annotations
  mmdb = lapply(mmdb, function(gs) {
    gids = geneIds(gs)
    gids = msigdb:::hcop$mouse_entrez_gene[msigdb:::hcop$human_entrez_gene %in% gids]
    gids = na.omit(unique(gids))
    geneIds(gs) = gids
    return(gs)
  })
  
  #create c5 category
  c5 = createC5OrgDb(org.Mm.egGO2ALLEGS, 'Mus musculus', old)
  
  #create c1 category
  c1 = createC1MmNCBI()
  
  #combine all and create Mm MSigDB
  mmdb = c(mmdb, c1, c5)
  mmdb = mmdb[order(sapply(mmdb, setName))]
  mmdb.all = ez2symMsigdb(mmdb, org.Mm.eg.db)
  
  return(mmdb.all)
}

ez2symMsigdb <- function(sigdb, orgdb) {
  #only retain genes with ids in the OrgDb
  allg = unique(keys(orgdb, 'ENTREZID'))
  sigdb = lapply(sigdb, function(gs) {
    geneIds(gs) = intersect(geneIds(gs), allg)
    return(gs)
  })
  
  #remove empty genesets
  sigdb = sigdb[sapply(lapply(sigdb, geneIds), length) > 0]
  sigdb = GeneSetCollection(sigdb)
  
  #convert to symbols
  gids = unique(unlist(geneIds(sigdb)))
  gmap = mapIds(orgdb, keys = gids, column = 'SYMBOL', keytype = 'ENTREZID')
  sigdb.sym = lapply(sigdb, function(gs) {
    geneIds(gs) = na.omit(unique(gmap[geneIds(gs)]))
    gs@geneIdType = SymbolIdentifier()
    return(gs)
  })
  sigdb.sym = GeneSetCollection(sigdb.sym)
  
  return(list('sym' = sigdb.sym, 'ezid' = sigdb))
}

#code adapted from Gordon K. Smyth and Alexandra Garnham (WEHI)
createC1MmNCBI <- function() {
  ncbi_url = 'https://ftp.ncbi.nlm.nih.gov/gene/DATA/GENE_INFO/Mammalia/Mus_musculus.gene_info.gz'
  
  #download file
  geneinfo_path = bfcrpath(bfc, ncbi_url)
  x = read.delim(geneinfo_path)
  
  # Extract cytoband string
  Loc = strsplit2(x$map_location, split = '\\|')
  i1 = grep(' [A-Z]', Loc[, 1])
  i2 = grep(' [A-Z]', Loc[, 2])
  Cytoband = rep('', nrow(Loc))
  Cytoband[i1] = Loc[i1, 1]
  Cytoband[i2] = Loc[i2, 2]
  
  # Remove empty strings
  i = Cytoband != ''
  GeneID = as.character(x$GeneID[i])
  Cytoband = Cytoband[i]
  
  # Split the cytoband string by space and '-'
  CB2 = strsplit2(Cytoband, split = '-')
  CB1 = strsplit2(CB2[, 1], split = ' ')
  Cytoband1 = paste(CB1[, 1], CB1[, 2], sep = 'q')
  Tab1 = data.frame(GeneID = GeneID, Cytoband = Cytoband1)
  
  # Genes in >1 cytoband
  i = grep('-', Cytoband)
  Cytoband2 = paste(CB1[i, 1], str_trim(CB2[i, 2]), sep = 'q')
  Tab2 = data.frame(GeneID = GeneID[i], Cytoband = Cytoband2)
  
  # Special case
  Tab3 = data.frame(GeneID = c('170942', '654820'), Cytoband = c('YqE', 'YqE'))
  
  # Make sets
  Tab = rbind(Tab1, Tab2, Tab3)
  
  #create geneset
  posgenes = split(Tab, Tab$Cytoband)
  c1 = lapply(posgenes, function (x) {
    gs = GeneSet(
      as.character(unique(x$GeneID)),
      setName = x$Cytoband[1],
      collectionType = BroadCollection(category = 'c1'),
      shortDescription = paste('Ensembl Genes in Cytogenetic Band', x$Cytoband[1]),
      organism = 'Mus musculus'
    )
    geneIdType(gs) = EntrezIdentifier()
    return(gs)
  })
  
  return(c1)
}

createC5OrgDb <- function(db, org, old) {
  gomap = as.list(db)
  gomap = lapply(gomap, as.character)
  
  #create c5 genesets using OrgDb
  c5 = mapply(
    function(genes, gsname, gstype, gsdesc) {
      go_prefix = ifelse(old, 'GO_', paste0('GO', gstype, '_'))
      gs = GeneSet(
        as.character(unique(genes)),
        setName = paste0(go_prefix, gsub(' ', '_', str_to_upper(gsname))),
        collectionType = BroadCollection(category = 'c5', subCategory = paste0('GO:', gstype)),
        shortDescription = gsdesc,
        organism = org
      )
      geneIdType(gs) = EntrezIdentifier()
      return(gs)
    },
    gomap,
    mapIds(GO.db, names(gomap), column = 'TERM', keytype = 'GOID'),
    mapIds(GO.db, names(gomap), column = 'ONTOLOGY', keytype = 'GOID'),
    mapIds(GO.db, names(gomap), column = 'DEFINITION', keytype = 'GOID'),
    SIMPLIFY = FALSE
  )
  
  return(c5)
}

createHCOPmap <- function() {
  hcop_url = 'https://ftp.ebi.ac.uk/pub/databases/genenames/hcop/human_mouse_hcop_fifteen_column.txt.gz'
  hcop_path = bfcrpath(bfc, hcop_url)
  
  #read mappings
  hcop = read.delim(hcop_path)
  hcop = unique(hcop[, c('mouse_entrez_gene', 'human_entrez_gene')])
  hcop = hcop[!hcop$mouse_entrez_gene %in% '-' & !hcop$human_entrez_gene %in% '-', ]
  
  return(hcop)
}

computeIdf <- function(msigGsc) {
  rmwords = vissE:::getMsigBlacklist()
  
  signames = sapply(msigGsc, GSEABase::setName)
  sigdesc_s = sapply(msigGsc, GSEABase::description)
  docs = list('Name' = signames, 'Short' = sigdesc_s)
  docs = lapply(docs, unique)
  
  #text-mining
  docs = lapply(docs, function(d) tm::Corpus(tm::VectorSource(d)))
  toSpace <- tm::content_transformer(function (x, pattern) gsub(pattern, " ", x))
  docs = lapply(docs, function(d) tm::tm_map(d, toSpace, "_"))
  docs = lapply(docs, function(d) tm::tm_map(d, toSpace, "/"))
  docs = lapply(docs, function(d) tm::tm_map(d, toSpace, "@"))
  docs = lapply(docs, function(d) tm::tm_map(d, toSpace, "\\|"))
  docs = lapply(docs, function(d) tm::tm_map(d, toSpace, "\\("))
  docs = lapply(docs, function(d) tm::tm_map(d, toSpace, "\\)"))
  
  # Convert the text to lower case
  docs = lapply(docs, function(d) tm::tm_map(d, tm::content_transformer(tolower)))
  # Remove numbers
  # docs = lapply(docs, function(d) tm_map(d, removeNumbers))
  # Remove english common stopwords
  docs = lapply(docs, function(d) tm::tm_map(d, tm::removeWords, tm::stopwords('english')))
  # Remove your own stop word
  # specify your stopwords as a character vector
  docs = lapply(docs, function(d) tm::tm_map(d, tm::removeWords, rmwords))
  # Remove punctuations
  docs = lapply(docs, function(d) tm::tm_map(d, tm::removePunctuation))
  # Eliminate extra white spaces
  docs = lapply(docs, function(d) tm::tm_map(d, tm::stripWhitespace))
  # Remove full numbers
  docs = lapply(docs, function(d) tm::tm_filter(d, function(x) !grepl('\\b[0-9]+\\b', x)))
  # Text lemmatisation
  docs = lapply(docs, function(d) tm::tm_map(d, textstem::lemmatize_strings))
  
  #compute idf
  dtms = lapply(docs, tm::TermDocumentMatrix)
  dtms = lapply(dtms, as.matrix)
  
  #compute IDF
  idfs = lapply(dtms, function(x) {
    idf = log(ncol(x) / rowSums(x != 0))
    return(idf)
  })
  #sort names to quicken searches
  idfs = lapply(idfs, function(x) {
    x[sort(names(x))]
  })
  
  return(idfs)
}

processMsigdbData <- function(msigdb_ver, old = FALSE) {
  bname = paste0('msigdb.v', msigdb_ver)
  
  #download data
  msigdb = getMsigdbData(msigdb_ver, old)
  
  #create human data
  msigdb.hs.SYM = msigdb[[1]]
  msigdb.hs.EZID = msigdb[[2]]
  saveRDS(msigdb.hs.SYM, file = paste0(bname, '.hs.SYM.rds'))
  saveRDS(msigdb.hs.EZID, file = paste0(bname, '.hs.EZID.rds'))
  
  #create mouse data
  msigdb.mm = createMmMsigdbData(msigdb.hs.EZID, old)
  msigdb.mm.SYM = msigdb.mm[[1]]
  msigdb.mm.EZID = msigdb.mm[[2]]
  saveRDS(msigdb.mm.SYM, file = paste0(bname, '.mm.SYM.rds'))
  saveRDS(msigdb.mm.EZID, file = paste0(bname, '.mm.EZID.rds'))
  
  #compute IDF
  idf_hs = computeIdf(msigdb.hs.EZID)
  idf_mm = computeIdf(msigdb.mm.EZID)
  saveRDS(idf_hs, file = paste0(bname, '.hs.idf.rds'))
  saveRDS(idf_mm, file = paste0(bname, '.mm.idf.rds'))
}

#----human-mouse homologs----
hcop = createHCOPmap()
usethis::use_data(hcop, internal = TRUE, overwrite = TRUE)

#----prepare MSigDB for the package----
cl = makeSOCKcluster(10, outfile = '')
registerDoSNOW(cl)

processMsigdbData('7.2', old = TRUE)
processMsigdbData('7.3')
processMsigdbData('7.4')
processMsigdbData('7.5')
processMsigdbData('7.5.1')

stopCluster(cl)

#' Retrieve MSigDB data hosted on the hub
#'
#' Download molecular signatures database (MSigDB) hosted on the ExperimentHub
#' or retrieve pre-downloaded version from cache. This package currently hosts
#' versions greater than 7.2 for human and mouse with both symbol and Entrez
#' identifiers.
#'
#' @param org a character, representing the organism whose signature database
#'   needs to be retrieved ("hs" for human and "mm" for mouse).
#' @param id a character, representing the ID type to use ("SYM" for gene
#'   symbols and "EZID" for Entrez IDs).
#' @param version a character, stating the version of MSigDB to be retrieved
#'   (should be >= 7.2). See `getMsigdbVersions()`.
#'
#' @return a GeneSetCollection, containing GeneSet objects from the specified
#'   version of the molecular signatures database (MSigDB).
#' @export
#'
#' @examples
#' gsc = getMsigdb('hs', 'SYM')
#' 
getMsigdb <- function(org = c('hs', 'mm'), id = c('SYM', 'EZID'), version = getMsigdbVersions()) {
  org = match.arg(org)
  id = match.arg(id)
  version = match.arg(version)
  
  #create object name
  obj_name = paste0('msigdb.v', version, '.', org, '.', id)
  gsc = getMSigdbObject(obj_name)
  
  return(gsc)
}

#' Retrieve IMEx PPI hosted on the hub
#'
#' Download International Molecular Exchange (IMEx) protein-protein interaction
#' (PPI) hosted on the ExperimentHub or retrieve pre-downloaded version from
#' cache. This package currently hosts versions for human and mouse with both
#' symbol and Entrez identifiers.
#'
#' @param org a character, representing the organism whose PPI database needs to
#'   be retrieved ("hs" for human and "mm" for mouse).
#' @param inferred a logical, indicating whether inference from other organisms
#'   should be included in the PPI.
#' @param version a character, stating the version of IMEX to be retrieved. See
#'   `getMsigdbVersions()`.
#'
#' @return a data.frame, containing the IMEx PPI.
#' @export
#'
#' @examples
#' imex = getIMEX("hs")
#'
getIMEX <- function(org = c("hs", "mm"), inferred = FALSE, version = getIMEXVersions()) {
  org = match.arg(org)
  version = match.arg(version)
  org = c("hs" = "9606", "mm" = "10090")[org]

  # create object name
  version = as.Date(version)
  obj_name = paste0("imex_hsmm_", format(version, "%m"), format(version, "%y"))
  imex = getMSigdbObject(obj_name)
  imex = imex[imex$Taxid %in% org & (imex$Inferred | inferred), ]
  imex = as.data.frame(imex)

  return(imex)
}

#' Retrieve MSigDB data hosted on the hub
#'
#' Download molecular signatures database (MSigDB) hosted on the ExperimentHub
#' or retrieve pre-downloaded version from cache. This package currently hosts
#' versions greater than 7.2 for human and mouse with both symbol and Entrez
#' identifiers.
#'
#' @param org a character, representing the organism whose signature database
#'   needs to be retrieved ("hs" for human and "mm" for mouse).
#' @param version a character, stating the version of MSigDB to be retrieved
#'   (should be >= 7.2). See `getMsigdbVersions()`.
#'
#' @return a list of named numeric vectors, containing inverse document frequency (IDF) weights. Names represent terms that the IDF is computed for. IDFs are computed using gene-set names ("Name") and short descriptions ("Short").
#' @export
#'
#' @examples
#' gsc = getMsigdbIDF("hs")
#'
getMsigdbIDF <- function(org = c('hs', 'mm'), version = getMsigdbVersions()) {
  org = match.arg(org)
  version = match.arg(version)
  
  #create object name
  obj_name = paste0('msigdb.v', version, '.', org, '.idf')
  idf = getMSigdbObject(obj_name)
  
  return(idf)
}

getMSigdbObject <- function(obj_name) {
  #load object
  eh = ExperimentHub::ExperimentHub()
  info = AnnotationHub::mcols(AnnotationHub::query(eh, 'msigdb'))
  id = rownames(info)[info$title %in% obj_name]
  if (length(id) != 1)
    stop('Data not found')
  
  return(suppressWarnings(eh[[id]]))
}

#' Subset collections and sub-collections of MSigDB
#'
#' The molecular signatures database (MSigDB) is composed of collections and
#' sub-collection. Many analyses (e.g. gene-set enrichment using limma::fry) are
#' best carried out within specific collections rather than across the entire
#' database of signatures. This function allows subsetting of MSigDB data
#' objects within this package using collection and sub-collection types.
#'
#' @param gsc a GeneSetCollection object, containing MSigDB genesets in the form
#'   of GeneSet objects.
#' @param collection a character, stating the collection(s) to be retrieved. The
#'   collection(s) must be one from the [listCollections()] function.
#' @param subcollection a character, stating the sub-collection(s) to be
#'   retrieved. The sub-collection(s) must be one from the
#'   [listSubCollections()] function.
#'
#' @inheritParams getMsigdb
#'
#' @return a GeneSetCollection object, containing gene sets belonging to the
#'   queries collection and/or sub-collection.
#' @export
#'
#' @examples
#' gsc = getMsigdb('hs', 'SYM')
#' subsetCollection(gsc, collection = "h")
#' 
subsetCollection <- function(gsc, collection = c(), subcollection = c()) {
  stopifnot(length(gsc) > 0)
  stopifnot(all(collection %in% listCollections(gsc)))
  stopifnot(all(subcollection %in% listSubCollections(gsc)))
  
  #filter collection & sub-collection
  ctype = lapply(gsc, GSEABase::collectionType)
  gsc = gsc[sapply(ctype, GSEABase::bcCategory) %in% collection |
              sapply(ctype, GSEABase::bcSubCategory) %in% subcollection]
  
  return(gsc)
}

#' List all collection types within a MSigDB gene set collection
#'
#' This function lists all the collection types present in a MSigDB gene set
#' collection. Descriptions of collections can be found at the MSigDB website.
#'
#' @inheritParams getMsigdb
#'
#' @return a character vector, containing character codes for all collections
#'   present in the GeneSetCollection object.
#' @export
#'
#' @examples
#' gsc = getMsigdb('hs', 'SYM')
#' listCollections(gsc)
#' 
listCollections <- function(gsc) {
  cat = unique(sapply(lapply(gsc, GSEABase::collectionType), GSEABase::bcCategory))
  cat = as.character(stats::na.omit(cat))
  return(cat)
}

#' List all sub-collection types within a MSigDB gene set collection
#'
#' This function lists all the sub-collection types present in a MSigDB gene set
#' collection. Descriptions of sub-collections can be found at the MSigDB
#' website.
#'
#' @inheritParams getMsigdb
#'
#' @return a character vector, containing character codes for all
#'   sub-collections present in the GeneSetCollection object.
#' @export
#'
#' @examples
#' gsc = getMsigdb('hs', 'SYM')
#' listSubCollections(gsc)
#' 
listSubCollections <- function(gsc) {
  subcat = unique(sapply(lapply(gsc, GSEABase::collectionType), GSEABase::bcSubCategory))
  subcat = as.character(stats::na.omit(subcat))
  return(subcat)
}

library(GSEABase)

test_that("list collections and sub-collections works", {
  msigdb = getMsigdb()
  
  #test empty
  expect_length(listCollections(GeneSetCollection(list())), 0)
  expect_length(listSubCollections(GeneSetCollection(list())), 0)
  
  #test non-broad sets
  nullgsc = GeneSetCollection(GeneSet(setName = 'A'))
  expect_error(listCollections(nullgsc))
  expect_error(listSubCollections(nullgsc))
  
  #test all
  expect_length(listCollections(msigdb), 9)
  expect_length(listSubCollections(msigdb), 18)
  
  #test Hallmarks
  hgsc = msigdb[grepl('HALLMARK', sapply(msigdb, setName))]
  expect_length(listCollections(hgsc), 1)
  expect_length(listSubCollections(hgsc), 0)
  
  #test GO
  gogsc = msigdb[grepl('^GO.._', sapply(msigdb, setName))]
  expect_length(listCollections(gogsc), 1)
  expect_length(listSubCollections(gogsc), 3)
})

test_that("subset collections works", {
  msigdb = getMsigdb()
  
  #test empty
  expect_error(subsetCollection(GeneSetCollection(list()), 'c1'))
  
  #test non-broad sets
  nullgsc = GeneSetCollection(GeneSet(setName = 'A'))
  expect_error(subsetCollection(nullgsc, 'c1'))
  
  expect_length(subsetCollection(msigdb, 'h'), 50)
  expect_length(subsetCollection(msigdb, 'c5'), sum(sapply(lapply(msigdb, collectionType), bcCategory) %in% 'c5'))
  expect_length(subsetCollection(msigdb, subcollection = 'GO:BP'), sum(sapply(lapply(msigdb, collectionType), bcSubCategory) %in% 'GO:BP'))
  expect_length(subsetCollection(msigdb, 'h', 'GO:BP'), sum(sapply(lapply(msigdb, collectionType), bcSubCategory) %in% 'GO:BP') + 50)
})

#' The Molecular Signatures Database (MSigDB)
#'
#' This ExperimentHub package contains gene expression signatures from the
#' molecular signatures database (MSigDB) for versions >= 7.2. Collections for
#' human and mouse are currently supported in this package. The mouse version
#' was developed in conjunction with Gordon K Smyth and Alexandra Garnham, and
#' reflects the collections available from WEHI (<https://bioinf.wehi.edu.au/MSigDB/>).
#'
#' The molecular signatures database (MSigDB) is a collection of over 25000 gene
#' expression signatures that are grouped into collections and sub-collections.
#' Metadata associated with signatures is collected and stored in the data in
#' this package.
#'
#' All data in this package are stored in a `GeneSetCollection` object from the
#' `GSEABase` package. Each gene expression signature in the collection is
#' stored in a `GeneSet` object from the `GSEABase` package. This data does not
#' include KEGG gene sets due to copyrights. Users can download this data using
#' functions provided in the package (see Details).
#'
#' @format A GeneSetCollection object composed of GeneSet objects representing
#'   all non-empty gene expression signatures from the molecular signatures
#'   database (MSigDB).
#' @details Data in this package does not include gene sets from the KEGG
#'   database due to licensing limitations. Users can use the [appendKEGG()]
#'   function in this package to download KEGG gene sets directly from the
#'   MSigDB and append to existing data objects.
#'
#'   The mouse MSigDB is created by translating human genes to mouse homologs
#'   using annotations from the Mouse Genome Informatics (MGI) database for most
#'   gene sets. Gene sets in the collections c1 (positional gene sets) and c5
#'   (ontologies) are recreated as information in these gene sets is organism
#'   specific. Positional gene sets are created using gene information from
#'   NCBI. Gene sets representing gene ontologies are derived from the mouse
#'   R/Bioconductor organism database (org.Mm.eg.db).
#' @references Subramanian, A., Tamayo, P., Mootha, V. K., Mukherjee, S., Ebert,
#'   B. L., Gillette, M. A., ... & Mesirov, J. P. (2005). Gene set enrichment
#'   analysis: a knowledge-based approach for interpreting genome-wide
#'   expression profiles. Proceedings of the National Academy of Sciences,
#'   102(43), 15545-15550.
#'
#'   Liberzon, A., Subramanian, A., Pinchback, R., Thorvaldsdóttir, H., Tamayo,
#'   P., & Mesirov, J. P. (2011). Molecular signatures database (MSigDB) 3.0.
#'   Bioinformatics, 27(12), 1739-1740.
#'
#'   Liberzon, A., Birger, C., Thorvaldsdóttir, H., Ghandi, M., Mesirov, J. P.,
#'   & Tamayo, P. (2015). The molecular signatures database hallmark gene set
#'   collection. Cell systems, 1(6), 417-425.
#' @section Acknowledgement: MSigDB is protected by copyright © 2004-2020 Broad
#'   Institute, Inc., Massachusetts Institute of Technology, and Regents of the
#'   University of California. Use of MSigDB is subject to the terms and
#'   conditions of the Creative Commons Attribution 4.0 International License -
#'   <https://creativecommons.org/licenses/by/4.0/>.
#'
#'   MSigDB gene sets derived from BioCarta pathways are the subject of
#'   copyright © 2000-2017 BioCarta, and are subject to Biocarta's Disclaimer of
#'   Liability and of Warranties -
#'   <https://data.broadinstitute.org/gsea-msigdb/msigdb/biocarta/biocarta_disclaimer_of_liability_and_of_warranties.txt>
#'
#' @examples
#' library(ExperimentHub)
#' eh <- ExperimentHub()
#' msigdb_datasets <- query(eh, "msigdb")
#'
#' #load data using different approaches
#' msigdb <- getMsigdb('hs', 'SYM')
#' msigdb <- eh[["EH5421"]]
#'
#' @name msigdb
#' @aliases msigdb-package
#'   
NULL

#' Get MSigDB versions included in the msigdb package
#'
#' @return a character, stating the MSigDB versions available in this package.
#' @export
#'
#' @examples
#' getMsigdbVersions()
#' 
getMsigdbVersions <- function() {
  fl = system.file("extdata", "metadata.csv", package = 'msigdb')
  meta = utils::read.csv(fl, stringsAsFactors = FALSE)
  versions = meta[grepl('^msigdb', meta$Title), 'SourceVersion']
  versions = as.character(unique(versions))
  versions = versions[order(-as.numeric(gsub('\\.', '', versions)))]
  
  return(versions)
}

#' Get IMEX versions included in the msigdb package
#'
#' @return a character, stating the IMEX versions available in this package.
#' @export
#'
#' @examples
#' getIMEXVersions()
#' 
getIMEXVersions <- function() {
  fl = system.file("extdata", "metadata.csv", package = 'msigdb')
  meta = utils::read.csv(fl, stringsAsFactors = FALSE)
  versions = meta[grepl('^imex', meta$Title), 'SourceVersion']
  versions = as.Date(versions)
  versions = as.character(sort(unique(versions), decreasing = TRUE))
  
  return(versions)
}

#retrieve file list that need to be updated
allfiles = list.files(pattern = '.rds$', recursive = TRUE)
old_meta = read.csv('inst/extdata/metadata.csv')
allfiles = setdiff(allfiles, basename(old_meta$RDataPath))
msigdb_files = basename(grep('msigdb', allfiles, value = TRUE))
imex_files = basename(grep('imex', allfiles, value = TRUE))
meta = c()

#----msigdb metadata----
#params
org_name = c('hs' = 'Homo sapiens', 'mm' = 'Mus musculus')
org_taxid = c('hs' = 9606, 'mm' = 10090)

#build description for each file
buildDescription <- function(ver, org, type) {
  if (type %in% 'SYM') {
    sprintf('Gene expression signatures (%s) from the Molecular Signatures Database (v%s) excluding KEGG gene sets. Signatures are represented using gene symbols.', org_name[org], ver)
  } else if (type %in% 'EZID') {
    sprintf('Gene expression signatures (%s) from the Molecular Signatures Database (v%s) excluding KEGG gene sets. Signatures are represented using Entrez IDs.', org_name[org], ver)
  } else {
    sprintf('This data is for internal use within vissE. Inverse document frequencies of words from gene expression signatures (%s) from the Molecular Signatures Database (v%s) excluding KEGG gene sets.', org_name[org], ver)
  }
}

if (length(msigdb_files) > 0) {
  msigdb_meta = plyr::ldply(msigdb_files, function(x) {
    #create regex
    fregex = 'msigdb.v([0-9]+\\.[0-9]+\\.?[0-9]?)\\.([a-z]+)\\.(.*).rds$'
    
    #extract file info
    ver = gsub(fregex, '\\1', x)
    org = gsub(fregex, '\\2', x)
    type = gsub(fregex, '\\3', x)
    
    #build record
    c(
      Title = gsub('.rds$', '', x),
      Description = buildDescription(ver, org, type),
      BiocVersion = as.numeric(as.character(BiocManager::version())) + 0.01,
      Genome = NA,
      SourceType = 'XML',
      SourceUrl = sprintf('https://data.broadinstitute.org/gsea-msigdb/msigdb/release/%s/msigdb_v%s.xml', ver, ver),
      SourceVersion = ver,
      Species = as.character(org_name[org]),
      TaxonomyId = as.numeric(org_taxid[org]),
      Coordinate_1_based = TRUE,
      DataProvider = 'Broad Institute',
      Maintainer = 'Dharmesh D. Bhuva <bhuva.d@wehi.edu.au>',
      RDataClass = 'GSEABase::GeneSetCollection',
      DispatchClass = 'Rds',
      RDataPath = file.path('msigdb', paste0('msigdb.v', ver), x)
    )
  })
  
  meta = rbind(meta, msigdb_meta)
}

#----IMEX metadata----
##build record
meta = rbind(meta, c(
  Title = 'imex_hsmm_0722',
  Description = 'Protein-protein interaction (PPI) network for human and mouse obtained from the international molecular exchange (IMEX) in July 2022.',
  BiocVersion = as.numeric(as.character(BiocManager::version())) + 0.01,
  Genome = NA,
  SourceType = 'TXT',
  SourceUrl = 'ftp://ftp.ebi.ac.uk/pub/databases/intact/2022-07-11/psimitab/intact-micluster.txt',
  SourceVersion = '2022-07-11',
  Species = 'Homo sapiens',
  TaxonomyId = '9606',
  Coordinate_1_based = TRUE,
  DataProvider = 'EBI',
  Maintainer = 'Dharmesh D. Bhuva <bhuva.d@wehi.edu.au>',
  RDataClass = 'data.frame',
  DispatchClass = 'Rds',
  RDataPath = file.path('msigdb', 'IMEx', 'imex_hsmm_0722.rds')
))

#write and test metadata file
meta = rbind(old_meta, meta)
write.csv(meta, file = 'inst/extdata/metadata.csv', row.names = FALSE)
ExperimentHubData::makeExperimentHubMetadata('../msigdb')

#' Add KEGG pathway derived gene sets to a collection
#'
#' This function adds gene sets derived from KEGG pathways to the MSigDB data
#' stored in this package. Direct user-end download from the MSigDB is required
#' to ensure KEGG licenses are adhered to.
#'
#' @inheritParams subsetCollection
#' @inheritParams getMsigdb
#' 
#' @return a GeneSetCollection object, storing gene sets from the MSigDB
#'   including the downloaded KEGG gene sets.
#' @export
#'
#' @examples
#' library(GSEABase)
#'
#' gs1 <- GeneSet(setName = 'gs1', geneIdType = SymbolIdentifier())
#' gsc <- GeneSetCollection(gs1)
#' gsc <- appendKEGG(gsc, version = '7.4)
#' gsc
#' 
appendKEGG <- function(gsc, version = getMsigdbVersions()) {
  version = match.arg(version)
  
  #get GeneSetCollection information
  idType = getMsigIdType(gsc)
  org = getMsigOrganism(gsc, idType)
  id = ifelse(methods::is(idType, 'SymbolIdentifier') & org %in% 'hs', 'symbols', 'entrez')
  
  # create URL
  link_ver = ifelse(grepl('^20', version), paste0(version, ".Hs"), version)
  fname = paste0("c2.cp.kegg.v", link_ver, ".", id, ".gmt")
  link = paste("https://data.broadinstitute.org/gsea-msigdb/msigdb/release", link_ver, fname, sep = "/")
  
  #download and process KEGG genesets
  gsc_kegg = GSEABase::getGmt(
    link,
    geneIdType = idType,
    collectionType = GSEABase::BroadCollection('c2', 'CP:KEGG')
  )
  
  if (methods::is(idType, 'EntrezIdentifier')) {
    gsc_kegg = lapply(gsc_kegg, function(gs) {
      gs@geneIdType = idType
      return(gs)
    })
    gsc_kegg = GSEABase::GeneSetCollection(gsc_kegg)
  }
  
  #conversions for mouse
  if (org %in% 'mm') {
    #convert Hs to Mm (Entrez IDs)
    gsc_kegg = lapply(gsc_kegg, function(gs) {
      gids = GSEABase::geneIds(gs)
      gids = hcop$mouse_entrez_gene[hcop$human_entrez_gene %in% gids]
      gids = stats::na.omit(unique(gids))
      GSEABase::geneIds(gs) = gids
      return(gs)
    })
    
    #only retain genes with ids in the OrgDb
    allg = unique(AnnotationDbi::keys(org.Mm.eg.db::org.Mm.eg.db, 'ENTREZID'))
    gsc_kegg = lapply(gsc_kegg, function(gs) {
      GSEABase::geneIds(gs) = intersect(GSEABase::geneIds(gs), allg)
      return(gs)
    })
    
    #convert to symbols if needed
    allg = unique(unlist(lapply(gsc_kegg, GSEABase::geneIds)))
    if (methods::is(idType, 'SymbolIdentifier')) {
      gmap = AnnotationDbi::mapIds(org.Mm.eg.db::org.Mm.eg.db,
                                   keys = allg,
                                   column = 'SYMBOL',
                                   keytype = 'ENTREZID')
      gsc_kegg = lapply(gsc_kegg, function(gs) {
        GSEABase::geneIds(gs) = stats::na.omit(unique(gmap[GSEABase::geneIds(gs)]))
        gs@geneIdType = GSEABase::SymbolIdentifier()
        return(gs)
      })
    }
  }
  
  #remove empty gene sets
  gsc_kegg = GSEABase::GeneSetCollection(gsc_kegg[sapply(lapply(gsc_kegg, GSEABase::geneIds), length) > 0])
  
  #annotate gene set with metadata
  gsc_kegg = GSEABase::GeneSetCollection(lapply(gsc_kegg, function(gs) {
    #add URLs
    GSEABase::urls(gs) = link
    return(gs)
  }))
  
  gsc = GSEABase::GeneSetCollection(c(gsc, gsc_kegg))
  
  return(gsc)
}


#' Infer organism type for the gene set collection
#'
#' Since both Human and Mouse MSigDB collections are hosted in this package,
#' this function infers the type of organism represented in a gene set
#' collection based on the gene IDs present. If not all gene IDs belong to the
#' same organism, the organism with more than 50% gene IDs present in the
#' collection is returned. In any other case, the function returns an error.
#'
#' @param idType a GSEABase::SymbolIdentifier or GSEABASE::EntrezIdentifier
#'   object, representing the ID type inferred from the [getMsigIdType()]
#'   function. Avoid providing this manually.
#' @inheritParams getMsigIdType
#'
#' @return a character, either "mm" (representing Mus musculus - mouse) or "hs"
#'   (representing Homo sapiens - human).
#' @export
#'
#' @examples
#' gsc <- getMsigdb()
#' id <- getMsigIdType(gsc)
#' getMsigOrganism(gsc, id)
#' 
getMsigOrganism <- function(gsc, idType) {
  #ensure ID types are the same in the collection
  keytype = ifelse(methods::is(idType, 'SymbolIdentifier'), 'SYMBOL', 'ENTREZID')
  
  #check gene IDs against organism databases
  allg = unique(unlist(GSEABase::geneIds(gsc)))
  
  if (all(allg %in% AnnotationDbi::keys(org.Hs.eg.db::org.Hs.eg.db, keytype))) {
    return(c('hs'))
  } else if (mean(allg %in% AnnotationDbi::keys(org.Hs.eg.db::org.Hs.eg.db, keytype)) > 0.5) {
    warning('Assuming the organism to be human.')
    return(c('hs'))
  } else if (all(allg %in% AnnotationDbi::keys(org.Mm.eg.db::org.Mm.eg.db, keytype))) {
    return(c('mm'))
  } else if (mean(allg %in% AnnotationDbi::keys(org.Mm.eg.db::org.Mm.eg.db, keytype)) > 0.5) {
    warning('Assuming the organism to be mouse.')
    return(c('mm'))
  } else {
    stop('Cannot infer organism.')
  }
}

#' Infer gene identifier type for the gene set collection
#'
#' The gene identifier (Symbol or Entrez ID) of a gene set collection is
#' inferred from the IDs present in the data. A collection should ideally store
#' gene sets using a single identifier type. This function returns the
#' identifier type (either SymbolIdentifier or EntrezIdentifier) of the
#' collection. It returns an error if the identifier is neither of these.
#'
#' @inheritParams subsetCollection
#'
#' @return a GSEABase::SymbolIdentifier or GSEABASE::EntrezIdentifier object,
#'   specifying the gene identifier type (gene symbols or Entrez IDs
#'   respectively).
#' @export
#'
#' @examples
#' gsc <- getMsigdb()
#' id <- getMsigIdType(gsc)
#' 
getMsigIdType <- function(gsc) {
  idType = sapply(gsc, function(gs) class(GSEABase::geneIdType(gs)))
  idType = unique(idType)
  if (length(idType) != 1) {
    stop('Gene identifiers should be the same in a collection.')
  }
  
  #create GSEABase ID type
  if (idType %in% 'SymbolIdentifier') {
    idType = GSEABase::SymbolIdentifier()
  } else if (idType %in% 'EntrezIdentifier'){
    idType = GSEABase::EntrezIdentifier()
  } else{
    stop('Only Symbol and Entrez identifiers supported.')
  }
  
  return(idType)
}

library(GSEABase)
library(org.Hs.eg.db)
library(org.Mm.eg.db)

test_that("appendKEGG works", {
  gs1 = GeneSet(setName = 'gs1', geneIdType = SymbolIdentifier())
  gs2 = GeneSet(setName = 'gs2', geneIdType = SymbolIdentifier())
  gsc = GeneSetCollection(c(gs1, gs2))
  expect_length(appendKEGG(gsc, '7.2'), 188)
})

test_that("ID type inference works", {
  gs1 = GeneSet(setName = 'gs1', geneIdType = SymbolIdentifier())
  gs1 = GeneSet(setName = 'gs1', geneIdType = SymbolIdentifier())
  gs2 = GeneSet(setName = 'gs2', geneIdType = EntrezIdentifier())
  gsc = GeneSetCollection(c(gs1, gs2))
  
  expect_s4_class(getMsigIdType(GeneSetCollection(gs1)), 'SymbolIdentifier')
  expect_s4_class(getMsigIdType(GeneSetCollection(gs2)), 'EntrezIdentifier')
  expect_error(getMsigIdType(gsc), 'identifiers should be the same')
  expect_error(getMsigIdType(gs1))
  
  gs1 = GeneSet(setName = 'gs1', geneIdType = NullIdentifier())
  gs2 = GeneSet(setName = 'gs2', geneIdType = NullIdentifier())
  gsc = GeneSetCollection(c(gs1, gs2))
  
  expect_error(getMsigIdType(gsc), 'identifiers supported')
})

test_that("Organism inference works", {
  gsmm = GeneSet(c('Esr1'), setName = 'gsmm', geneIdType = SymbolIdentifier())
  gshs = GeneSet(c('ESR1'), setName = 'gshs', geneIdType = SymbolIdentifier())
  gsmm_e = gsmm
  geneIdType(gsmm_e) = EntrezIdentifier('org.Mm.eg.db')
  setName(gsmm_e) = 'gsmm_e'
  gshs_e = gshs
  setName(gshs_e) = 'gshs_e'
  geneIdType(gshs_e) = EntrezIdentifier('org.Hs.eg.db')
  
  expect_equal(getMsigOrganism(GeneSetCollection(gshs), getMsigIdType(GeneSetCollection(gshs))), 'hs')
  expect_equal(getMsigOrganism(GeneSetCollection(gshs_e), getMsigIdType(GeneSetCollection(gshs_e))), 'hs')
  expect_equal(getMsigOrganism(GeneSetCollection(gsmm), getMsigIdType(GeneSetCollection(gsmm))), 'mm')
  expect_equal(getMsigOrganism(GeneSetCollection(gsmm_e), getMsigIdType(GeneSetCollection(gsmm_e))), 'mm')
  
  expect_error(getMsigOrganism(GeneSetCollection(gshs, gsmm), getMsigIdType(GeneSetCollection(gsmm))))
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  message = FALSE,
  comment = "#>"
)

## ----install, eval=FALSE------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("msigdb")

## ----load-packages, message=FALSE---------------------------------------------
library(msigdb)
library(ExperimentHub)
library(GSEABase)

## ----get-msigdb---------------------------------------------------------------
eh = ExperimentHub()
query(eh , 'msigdb')

## ----download-msigdb-sym-id---------------------------------------------------
eh[['EH5421']]

## ----download-msigdb-sym-getMsigdb--------------------------------------------
#use the custom accessor to select a specific version of MSigDB
msigdb.hs = getMsigdb(org = 'hs', id = 'SYM', version = '7.4')
msigdb.hs

## ----append-kegg--------------------------------------------------------------
msigdb.hs = appendKEGG(msigdb.hs)
msigdb.hs

## ----process-gsc--------------------------------------------------------------
length(msigdb.hs)

## ----access-gs----------------------------------------------------------------
gs = msigdb.hs[[1000]]
gs
#get genes in the signature
geneIds(gs)
#get collection type
collectionType(gs)
#get MSigDB category
bcCategory(collectionType(gs))
#get MSigDB subcategory
bcSubCategory(collectionType(gs))
#get description
description(gs)
#get details
details(gs)

## ----summarise-gsc------------------------------------------------------------
#calculate the number of signatures in each category
table(sapply(lapply(msigdb.hs, collectionType), bcCategory))
#calculate the number of signatures in each subcategory
table(sapply(lapply(msigdb.hs, collectionType), bcSubCategory))
#plot the distribution of sizes
hist(sapply(lapply(msigdb.hs, geneIds), length),
     main = 'MSigDB signature size distribution',
     xlab = 'Signature size')

## ----list-collections---------------------------------------------------------
listCollections(msigdb.hs)
listSubCollections(msigdb.hs)

## -----------------------------------------------------------------------------
#retrieeve the hallmarks gene sets
subsetCollection(msigdb.hs, 'h')
#retrieve the biological processes category of gene ontology
subsetCollection(msigdb.hs, 'c5', 'GO:BP')

## ----load-limma, message=FALSE------------------------------------------------
library(limma)

#create expression data
allg = unique(unlist(geneIds(msigdb.hs)))
emat = matrix(0, nrow = length(allg), ncol = 6)
rownames(emat) = allg
colnames(emat) = paste0('sample', 1:6)
head(emat)

## ----subset-msigdb------------------------------------------------------------
#retrieve collections
hallmarks = subsetCollection(msigdb.hs, 'h')
msigdb_ids = geneIds(hallmarks)

#convert gene sets into a list of gene indices
fry_indices = ids2indices(msigdb_ids, rownames(emat))
fry_indices[1:2]

## ----download-msig-sym-id-mouse-----------------------------------------------
msigdb.mm = getMsigdb(org = 'mm', id = 'SYM', version = '7.4')
msigdb.mm

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

---
title: "msigdb: The molecular signatures database (MSigDB) in R"
author: "Dharmesh D. Bhuva"
date: "`r BiocStyle::doc_date()`"
output:
  prettydoc::html_pretty:
    theme: hpstr
    toc: yes
    toc_depth: 2
    number_sections: yes
    fig_caption: yes
    df_print: paged
vignette: >
  %\VignetteIndexEntry{msigdb}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  message = FALSE,
  comment = "#>"
)
```

# The Molecular Signatures Database (MSigDB)

The [molecular signatures database (MSigDB)](https://www.gsea-msigdb.org/gsea/msigdb) is one of the largest collections of molecular signatures or gene expression signatures. A variety of gene expression signatures are hosted on this database including experimentally derived signatures and signatures representing pathways and ontologies from other curated databases. This rich collection of gene expression signatures (\>25,000) can facilitate a wide variety of signature-based analyses, the most popular being gene set enrichment analyses. These signatures can be used to perform enrichment analysis in a DE experiment using tools such as GSEA, fry (from limma) and camera (from limma). Alternatively, they can be used to perform single-sample gene-set analysis of individual transcriptomic profiles using approaches such as [singscore](https://doi.org/doi:10.18129/B9.bioc.singscore), ssGSEA and [GSVA](https://doi.org/doi:10.18129/B9.bioc.GSVA).

This package provides the gene sets in the MSigDB in the form of `GeneSet` objects. This data structure is specifically designed to store information about gene sets, including their member genes and metadata. Other packages, such as `msigdbr` and `EGSEAdata` provide these gene sets too, however, they do so by storing them as lists or tibbles. These structures are not specific to gene sets therefore do not allow storage of important metadata associated with each gene set, for example, their short and long descriptions. Additionally, the lack of structure allows creation of invalid gene sets. Accessory functions implemented in the `GSEABase` package provide a neat interface to interact with `GeneSet` objects.

This package can be installed using the code below:

```{r install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("msigdb")
```

# Download data from the msigdb R package

This ExperimentHub package processes the latest version of the MSigDB database into R objects that can be queried using the [GSEABase](https://doi.org/doi:10.18129/B9.bioc.GSEABase) R/Bioconductor package. The entire database is stored in a `GeneSetCollection` object which in turn stores each signature as a `GeneSet` object. All empty gene expression signatures (i.e. no genes formed the signature) have been dropped. Data in this package can be downloaded using the `ExperimentHub` interface as shown below.

To download the data, we first need to get a list of the data available in the `msigdb` package and determine the unique identifiers for each data. The `query()` function assists in getting this list.

```{r load-packages, message=FALSE}
library(msigdb)
library(ExperimentHub)
library(GSEABase)
```

```{r get-msigdb}
eh = ExperimentHub()
query(eh , 'msigdb')
```

Data can then be downloaded using the unique identifier.

```{r download-msigdb-sym-id}
eh[['EH5421']]
```

Data can also be downloaded using the custom accessor \`msigdb::getMsigdb()\`:

```{r download-msigdb-sym-getMsigdb}
#use the custom accessor to select a specific version of MSigDB
msigdb.hs = getMsigdb(org = 'hs', id = 'SYM', version = '7.4')
msigdb.hs
```

# Downloading and integrating KEGG gene sets

KEGG gene sets cannot be integrated within this ExperimentHub package due to licensing limitations. However, users can download, process and integrate the data directly from the MSigDB when needed. This can be done using the code that follows.

```{r append-kegg}
msigdb.hs = appendKEGG(msigdb.hs)
msigdb.hs
```

# Accessing the GeneSet and GeneSetCollection objects

A GeneSetCollection object is effectively a list therefore all list processing functions such as `length` and `lapply` can be used to process its constituents

```{r process-gsc}
length(msigdb.hs)
```

Each signature is stored in a `GeneSet` object and can be processed using functions in the `GSEABase` R/Bioconductor package.

```{r access-gs}
gs = msigdb.hs[[1000]]
gs
#get genes in the signature
geneIds(gs)
#get collection type
collectionType(gs)
#get MSigDB category
bcCategory(collectionType(gs))
#get MSigDB subcategory
bcSubCategory(collectionType(gs))
#get description
description(gs)
#get details
details(gs)
```

We can also summarise some of these values across the entire database. Description of these codes can be found at the MSigDB website (<https://www.gsea-msigdb.org/gsea/msigdb>).

```{r summarise-gsc}
#calculate the number of signatures in each category
table(sapply(lapply(msigdb.hs, collectionType), bcCategory))
#calculate the number of signatures in each subcategory
table(sapply(lapply(msigdb.hs, collectionType), bcSubCategory))
#plot the distribution of sizes
hist(sapply(lapply(msigdb.hs, geneIds), length),
     main = 'MSigDB signature size distribution',
     xlab = 'Signature size')
```

# Subset collections from the MSigDB

Most gene set analysis is performed within specific collections rather than across the entire database. This package comes with functions to subset specific collections. The list of all collections and sub-collections present within a GeneSetCollection object can be listed using the functions below:

```{r list-collections}
listCollections(msigdb.hs)
listSubCollections(msigdb.hs)
```

Specific collections can be retrieved using the code below:

```{r}
#retrieeve the hallmarks gene sets
subsetCollection(msigdb.hs, 'h')
#retrieve the biological processes category of gene ontology
subsetCollection(msigdb.hs, 'c5', 'GO:BP')
```

# Preparing collections for limma::fry

Any gene-set collection can be easily transformed for usage with `limma::fry` by first transforming it into a list of gene IDs and following that with a transformation to indices as shown below.

```{r load-limma, message=FALSE}
library(limma)

#create expression data
allg = unique(unlist(geneIds(msigdb.hs)))
emat = matrix(0, nrow = length(allg), ncol = 6)
rownames(emat) = allg
colnames(emat) = paste0('sample', 1:6)
head(emat)
```

```{r subset-msigdb}
#retrieve collections
hallmarks = subsetCollection(msigdb.hs, 'h')
msigdb_ids = geneIds(hallmarks)

#convert gene sets into a list of gene indices
fry_indices = ids2indices(msigdb_ids, rownames(emat))
fry_indices[1:2]
```

# Accessing the mouse MSigDB

The mouse MSigDB has been created in collaboration with Gordon K. Smyth and Alex Garnham from WEHI. The code they use to generate the mouse MSigDB has been used in this package. Detailed description of the steps conducted to convert human gene expression signatures to mouse can be found at <http://bioinf.wehi.edu.au/MSigDB>. Mouse homologs for human genes were obtained using the HCOP database (as of 18/03/2021).

All the above functions apply to the mouse MSigDB and can be used to interact with the collection.

```{r download-msig-sym-id-mouse}
msigdb.mm = getMsigdb(org = 'mm', id = 'SYM', version = '7.4')
msigdb.mm
```

# Session information

```{r sessionInfo}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addKEGG.R
\name{getMsigIdType}
\alias{getMsigIdType}
\title{Infer gene identifier type for the gene set collection}
\usage{
getMsigIdType(gsc)
}
\arguments{
\item{gsc}{a GeneSetCollection object, containing MSigDB genesets in the form
of GeneSet objects.}
}
\value{
a GSEABase::SymbolIdentifier or GSEABASE::EntrezIdentifier object,
specifying the gene identifier type (gene symbols or Entrez IDs
respectively).
}
\description{
The gene identifier (Symbol or Entrez ID) of a gene set collection is
inferred from the IDs present in the data. A collection should ideally store
gene sets using a single identifier type. This function returns the
identifier type (either SymbolIdentifier or EntrezIdentifier) of the
collection. It returns an error if the identifier is neither of these.
}
\examples{
gsc <- getMsigdb()
id <- getMsigIdType(gsc)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessMSigDB.R
\name{subsetCollection}
\alias{subsetCollection}
\title{Subset collections and sub-collections of MSigDB}
\usage{
subsetCollection(gsc, collection = c(), subcollection = c())
}
\arguments{
\item{gsc}{a GeneSetCollection object, containing MSigDB genesets in the form
of GeneSet objects.}

\item{collection}{a character, stating the collection(s) to be retrieved. The
collection(s) must be one from the \code{\link[=listCollections]{listCollections()}} function.}

\item{subcollection}{a character, stating the sub-collection(s) to be
retrieved. The sub-collection(s) must be one from the
\code{\link[=listSubCollections]{listSubCollections()}} function.}
}
\value{
a GeneSetCollection object, containing gene sets belonging to the
queries collection and/or sub-collection.
}
\description{
The molecular signatures database (MSigDB) is composed of collections and
sub-collection. Many analyses (e.g. gene-set enrichment using limma::fry) are
best carried out within specific collections rather than across the entire
database of signatures. This function allows subsetting of MSigDB data
objects within this package using collection and sub-collection types.
}
\examples{
gsc = getMsigdb('hs', 'SYM')
subsetCollection(gsc, collection = "h")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessMSigDB.R
\name{getMsigdbIDF}
\alias{getMsigdbIDF}
\title{Retrieve MSigDB data hosted on the hub}
\usage{
getMsigdbIDF(org = c("hs", "mm"), version = getMsigdbVersions())
}
\arguments{
\item{org}{a character, representing the organism whose signature database
needs to be retrieved ("hs" for human and "mm" for mouse).}

\item{version}{a character, stating the version of MSigDB to be retrieved
(should be >= 7.2). See \code{getMsigdbVersions()}.}
}
\value{
a list of named numeric vectors, containing inverse document frequency (IDF) weights. Names represent terms that the IDF is computed for. IDFs are computed using gene-set names ("Name") and short descriptions ("Short").
}
\description{
Download molecular signatures database (MSigDB) hosted on the ExperimentHub
or retrieve pre-downloaded version from cache. This package currently hosts
versions greater than 7.2 for human and mouse with both symbol and Entrez
identifiers.
}
\examples{
gsc = getMsigdbIDF("hs")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{getMsigdbVersions}
\alias{getMsigdbVersions}
\title{Get MSigDB versions included in the msigdb package}
\usage{
getMsigdbVersions()
}
\value{
a character, stating the MSigDB versions available in this package.
}
\description{
Get MSigDB versions included in the msigdb package
}
\examples{
getMsigdbVersions()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessMSigDB.R
\name{getMsigdb}
\alias{getMsigdb}
\title{Retrieve MSigDB data hosted on the hub}
\usage{
getMsigdb(
  org = c("hs", "mm"),
  id = c("SYM", "EZID"),
  version = getMsigdbVersions()
)
}
\arguments{
\item{org}{a character, representing the organism whose signature database
needs to be retrieved ("hs" for human and "mm" for mouse).}

\item{id}{a character, representing the ID type to use ("SYM" for gene
symbols and "EZID" for Entrez IDs).}

\item{version}{a character, stating the version of MSigDB to be retrieved
(should be >= 7.2). See \code{getMsigdbVersions()}.}
}
\value{
a GeneSetCollection, containing GeneSet objects from the specified
version of the molecular signatures database (MSigDB).
}
\description{
Download molecular signatures database (MSigDB) hosted on the ExperimentHub
or retrieve pre-downloaded version from cache. This package currently hosts
versions greater than 7.2 for human and mouse with both symbol and Entrez
identifiers.
}
\examples{
gsc = getMsigdb('hs', 'SYM')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addKEGG.R
\name{getMsigOrganism}
\alias{getMsigOrganism}
\title{Infer organism type for the gene set collection}
\usage{
getMsigOrganism(gsc, idType)
}
\arguments{
\item{gsc}{a GeneSetCollection object, containing MSigDB genesets in the form
of GeneSet objects.}

\item{idType}{a GSEABase::SymbolIdentifier or GSEABASE::EntrezIdentifier
object, representing the ID type inferred from the \code{\link[=getMsigIdType]{getMsigIdType()}}
function. Avoid providing this manually.}
}
\value{
a character, either "mm" (representing Mus musculus - mouse) or "hs"
(representing Homo sapiens - human).
}
\description{
Since both Human and Mouse MSigDB collections are hosted in this package,
this function infers the type of organism represented in a gene set
collection based on the gene IDs present. If not all gene IDs belong to the
same organism, the organism with more than 50\% gene IDs present in the
collection is returned. In any other case, the function returns an error.
}
\examples{
gsc <- getMsigdb()
id <- getMsigIdType(gsc)
getMsigOrganism(gsc, id)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{getIMEXVersions}
\alias{getIMEXVersions}
\title{Get IMEX versions included in the msigdb package}
\usage{
getIMEXVersions()
}
\value{
a character, stating the IMEX versions available in this package.
}
\description{
Get IMEX versions included in the msigdb package
}
\examples{
getIMEXVersions()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessMSigDB.R
\name{getIMEX}
\alias{getIMEX}
\title{Retrieve IMEx PPI hosted on the hub}
\usage{
getIMEX(org = c("hs", "mm"), inferred = FALSE, version = getIMEXVersions())
}
\arguments{
\item{org}{a character, representing the organism whose PPI database needs to
be retrieved ("hs" for human and "mm" for mouse).}

\item{inferred}{a logical, indicating whether inference from other organisms
should be included in the PPI.}

\item{version}{a character, stating the version of IMEX to be retrieved. See
\code{getMsigdbVersions()}.}
}
\value{
a data.frame, containing the IMEx PPI.
}
\description{
Download International Molecular Exchange (IMEx) protein-protein interaction
(PPI) hosted on the ExperimentHub or retrieve pre-downloaded version from
cache. This package currently hosts versions for human and mouse with both
symbol and Entrez identifiers.
}
\examples{
imex = getIMEX("hs")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessMSigDB.R
\name{listCollections}
\alias{listCollections}
\title{List all collection types within a MSigDB gene set collection}
\usage{
listCollections(gsc)
}
\value{
a character vector, containing character codes for all collections
present in the GeneSetCollection object.
}
\description{
This function lists all the collection types present in a MSigDB gene set
collection. Descriptions of collections can be found at the MSigDB website.
}
\examples{
gsc = getMsigdb('hs', 'SYM')
listCollections(gsc)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{msigdb}
\alias{msigdb}
\alias{msigdb-package}
\title{The Molecular Signatures Database (MSigDB)}
\format{
A GeneSetCollection object composed of GeneSet objects representing
all non-empty gene expression signatures from the molecular signatures
database (MSigDB).
}
\description{
This ExperimentHub package contains gene expression signatures from the
molecular signatures database (MSigDB) for versions >= 7.2. Collections for
human and mouse are currently supported in this package. The mouse version
was developed in conjunction with Gordon K Smyth and Alexandra Garnham, and
reflects the collections available from WEHI (\url{https://bioinf.wehi.edu.au/MSigDB/}).
}
\details{
The molecular signatures database (MSigDB) is a collection of over 25000 gene
expression signatures that are grouped into collections and sub-collections.
Metadata associated with signatures is collected and stored in the data in
this package.

All data in this package are stored in a \code{GeneSetCollection} object from the
\code{GSEABase} package. Each gene expression signature in the collection is
stored in a \code{GeneSet} object from the \code{GSEABase} package. This data does not
include KEGG gene sets due to copyrights. Users can download this data using
functions provided in the package (see Details).

Data in this package does not include gene sets from the KEGG
database due to licensing limitations. Users can use the \code{\link[=appendKEGG]{appendKEGG()}}
function in this package to download KEGG gene sets directly from the
MSigDB and append to existing data objects.

The mouse MSigDB is created by translating human genes to mouse homologs
using annotations from the Mouse Genome Informatics (MGI) database for most
gene sets. Gene sets in the collections c1 (positional gene sets) and c5
(ontologies) are recreated as information in these gene sets is organism
specific. Positional gene sets are created using gene information from
NCBI. Gene sets representing gene ontologies are derived from the mouse
R/Bioconductor organism database (org.Mm.eg.db).
}
\section{Acknowledgement}{
 MSigDB is protected by copyright © 2004-2020 Broad
Institute, Inc., Massachusetts Institute of Technology, and Regents of the
University of California. Use of MSigDB is subject to the terms and
conditions of the Creative Commons Attribution 4.0 International License -
\url{https://creativecommons.org/licenses/by/4.0/}.

MSigDB gene sets derived from BioCarta pathways are the subject of
copyright © 2000-2017 BioCarta, and are subject to Biocarta's Disclaimer of
Liability and of Warranties -
\url{https://data.broadinstitute.org/gsea-msigdb/msigdb/biocarta/biocarta_disclaimer_of_liability_and_of_warranties.txt}
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
msigdb_datasets <- query(eh, "msigdb")

#load data using different approaches
msigdb <- getMsigdb('hs', 'SYM')
msigdb <- eh[["EH5421"]]

}
\references{
Subramanian, A., Tamayo, P., Mootha, V. K., Mukherjee, S., Ebert,
B. L., Gillette, M. A., ... & Mesirov, J. P. (2005). Gene set enrichment
analysis: a knowledge-based approach for interpreting genome-wide
expression profiles. Proceedings of the National Academy of Sciences,
102(43), 15545-15550.

Liberzon, A., Subramanian, A., Pinchback, R., Thorvaldsdóttir, H., Tamayo,
P., & Mesirov, J. P. (2011). Molecular signatures database (MSigDB) 3.0.
Bioinformatics, 27(12), 1739-1740.

Liberzon, A., Birger, C., Thorvaldsdóttir, H., Ghandi, M., Mesirov, J. P.,
& Tamayo, P. (2015). The molecular signatures database hallmark gene set
collection. Cell systems, 1(6), 417-425.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addKEGG.R
\name{appendKEGG}
\alias{appendKEGG}
\title{Add KEGG pathway derived gene sets to a collection}
\usage{
appendKEGG(gsc, version = getMsigdbVersions())
}
\arguments{
\item{gsc}{a GeneSetCollection object, containing MSigDB genesets in the form
of GeneSet objects.}

\item{version}{a character, stating the version of MSigDB to be retrieved
(should be >= 7.2). See \code{getMsigdbVersions()}.}
}
\value{
a GeneSetCollection object, storing gene sets from the MSigDB
including the downloaded KEGG gene sets.
}
\description{
This function adds gene sets derived from KEGG pathways to the MSigDB data
stored in this package. Direct user-end download from the MSigDB is required
to ensure KEGG licenses are adhered to.
}
